DECLARE SUB SendTime (Frames&)
DECLARE SUB PrintError (Msg$)
DECLARE SUB SendCommand (Cmd$)
DECLARE SUB ReadFrameCount ()
DECLARE SUB ReadUserBits ()
DECLARE SUB Delay (Count!)

'Initialize the program as needed:
COMMON SHARED TimeoutCode%, Frames&, Box%
COMMON SHARED GetTime$, GoodTime$, OtherTime$
COMMON SHARED GetUserBits$, GoodUB$, OtherUB$
Box% = FREEFILE                 'Get a file number for the box.
OPEN "COM1:9600,N,8,1" FOR RANDOM AS #Box%
'OPEN "COM2:9600,N,8,1" FOR RANDOM AS #Box%

'Define all VTR command and response codes to be used:
BCDIO$ = CHR$(176)
BinaryIO$ = CHR$(177)
GetTime$ = CHR$(224)
GetUserBits$ = CHR$(226)
StopVTR$ = CHR$(128)
PauseVTR$ = CHR$(129)
PlayVTR$ = CHR$(130)
GotoFrame$ = CHR$(112)
EjectTape$ = CHR$(147)
FastFwdVTR$ = CHR$(132)
RewindVTR$ = CHR$(133)
ShtlFwdVTR$ = CHR$(64)
ShtlRevVTR$ = CHR$(65)
SlowSpeed$ = CHR$(42)    '1/25 => 19;  1/10 => 32;  1/5 => 42
StepFwdVTR$ = CHR$(134)
StepRevVTR$ = CHR$(135)
GoodTime$ = CHR$(96)
OtherTime$ = CHR$(97)
GoodUB$ = CHR$(98)
OtherUB$ = CHR$(99)

'Define special key codes:
Null$ = CHR$(0)
EscKey$ = CHR$(27)
RightArrow$ = Null$ + CHR$(77)
LeftArrow$ = Null$ + CHR$(75)

'Print the main menu screen:
CLS
LOCATE 3, 18: PRINT "Adrienne Electronics Corporation"
LOCATE 4, 28: PRINT "AEC-BOX-80"
LOCATE 5, 18: PRINT "VTR Control Demonstration Program"
LOCATE 7, 23: PRINT "Press S for Stop"
LOCATE 8, 23: PRINT "Press P for Play"
LOCATE 9, 23: PRINT "Press F for Fast Forward"
LOCATE 10, 23: PRINT "Press R for Rewind"
LOCATE 11, 23: PRINT "Press 0 for Pause"
LOCATE 12, 23: PRINT "Press E for Eject"
LOCATE 13, 23: PRINT "Press G for GoTo"
LOCATE 14, 23: PRINT "Press > for Slow Forward"
LOCATE 15, 23: PRINT "Press < for Slow Reverse"
LOCATE 16, 23: PRINT "Press "; CHR$(26); " for Step Forward"
LOCATE 17, 23: PRINT "Press "; CHR$(27); " for Step Reverse"
LOCATE 18, 23: PRINT "Press Esc to exit..."

'Send setup commands to the VTR:
SendCommand BinaryIO$

'Begin the main program loop:
DO
   Key$ = INKEY$                        'Read next keypress, if any.
   Key$ = UCASE$(Key$)                  'Convert all letters to upper case.
   SELECT CASE Key$
      CASE EscKey$                      'Do nothing.
      CASE "S"
         SendCommand StopVTR$
      CASE "P"
         SendCommand PlayVTR$
      CASE "0"
         SendCommand PauseVTR$
      CASE "F"
         SendCommand FastFwdVTR$
      CASE "R"
         SendCommand RewindVTR$
      CASE ">"
         SendCommand ShtlFwdVTR$
         SendCommand SlowSpeed$         'Send speed code too.
      CASE "<"
         SendCommand ShtlRevVTR$
         SendCommand SlowSpeed$         'Send speed code too.
      CASE "G"
         LOCATE 1, 1
         INPUT "Go to which frame"; Frames&
         LOCATE 1, 1
         PRINT SPACE$(40);              'Erase the prompt.
         SendCommand GotoFrame$
         SendTime Frames&
      CASE "E"
         SendCommand EjectTape$
      CASE LeftArrow$
         SendCommand StepRevVTR$
      CASE RightArrow$
         SendCommand StepFwdVTR$
      CASE ELSE
         ReadFrameCount                    'Read and display the frame count.
         SendCommand BCDIO$                'Select packed BCD I/O.
         ReadUserBits
         SendCommand BinaryIO$             'Reselect binary I/O.
      END SELECT
   LOOP UNTIL Key$ = EscKey$               'End program when get ESC key.
CLS
END


NoResponse:
TIMER OFF
PrintError "No response from box."
TimeoutCode% = 1
RETURN


SUB Delay (Count!)
   StartTime! = TIMER
   DO WHILE ABS(TIMER - StartTime!) < Count!    'Wait as desired.
      LOOP
END SUB


SUB PrintError (Msg$)
   LOCATE 1, 1
   PRINT Msg$;
   Delay 1!                     'Wait for one second.
   LOCATE 1, 1
   PRINT SPACE$(60);            'Erase the message.
   Delay 1!                     'Wait for one second.
END SUB


SUB ReadFrameCount
   STATIC BadTCCounter%
   PRINT #Box%, GetTime$;                    'Send the command byte.
   TimeoutCode% = 0
   TIMER ON
   ON TIMER(1) GOSUB NoResponse              'Set for 1 second timeout.
   DO WHILE (TimeoutCode% = 0) AND LOC(Box%) < 6  'Wait for data or timeout.
      LOOP
   TIMER OFF
   IF NOT EOF(Box%) THEN
      Msg$ = INPUT$(LOC(Box%), #Box%)          'Read the entire message.
      IF LEN(Msg$) = 6 THEN
         LOCATE 23, 23
         PRINT "Frames Count =  ";
         Rsp$ = MID$(Msg$, 1, 1)               'Get response code.
         Frames& = CVL(MID$(Msg$, 2, 4))       'Read frame count.
         PRINT USING "#######"; Frames&;
         IF Rsp$ = GoodTime$ THEN              'See if frame count is OK.
            PRINT " ";
            BadTCCounter% = 0
         ELSE
            BadTCCounter% = BadTCCounter% + 1
            IF BadTCCounter% > 100 THEN
               PRINT "?";
            END IF
         END IF
      ELSE
         PrintError "Frame count message error detected"
      END IF
   END IF
END SUB


SUB ReadUserBits
   STATIC BadUBCounter%
   PRINT #Box%, GetUserBits$;                  'Send the command byte.
   TimeoutCode% = 0
   TIMER ON
   ON TIMER(1) GOSUB NoResponse                'Set for 1 second timeout.
   DO WHILE (TimeoutCode% = 0) AND LOC(Box%) < 6  'Wait for data or timeout.
      LOOP
   TIMER OFF
   IF NOT EOF(Box%) THEN
      Msg$ = INPUT$(LOC(Box%), #Box%)          'Read the entire message.
      IF LEN(Msg$) = 6 THEN
         LOCATE 25, 23
         PRINT "User Bits = ";
         Rsp$ = MID$(Msg$, 1, 1)               'Get response code.
         PRINT RIGHT$("00" + HEX$(ASC(MID$(Msg$, 2, 1))), 2); 'Print Hours.
         PRINT ":";
         PRINT RIGHT$("00" + HEX$(ASC(MID$(Msg$, 3, 1))), 2); 'Print Minutes.
         PRINT ":";
         PRINT RIGHT$("00" + HEX$(ASC(MID$(Msg$, 4, 1))), 2); 'Print Seconds.
         PRINT ":";
         PRINT RIGHT$("00" + HEX$(ASC(MID$(Msg$, 5, 1))), 2); 'Print Frames.
         IF Rsp$ = GoodUB$ THEN              'See if frame count is OK.
            PRINT " ";
            BadUBCounter% = 0
         ELSE
            BadUBCounter% = BadUBCounter% + 1
            IF BadUBCounter% > 100 THEN
               PRINT "?";
            END IF
         END IF
      ELSE
         PrintError "User bits message error detected"
      END IF
   END IF
END SUB


SUB SendCommand (Cmd$)
   PRINT #Box%, Cmd$;                     'Send the command byte to the box.
   TimeoutCode% = 0
   TIMER ON
   ON TIMER(1) GOSUB NoResponse            'Set for 1 second timeout.
   DO WHILE (TimeoutCode% = 0) AND EOF(Box%)  'Wait for data or timeout.
      LOOP
   TIMER OFF
   IF NOT EOF(Box%) THEN
      Rsp$ = INPUT$(1, #Box%)              'Read returned byte.
      IF Rsp$ <> Cmd$ THEN                 'Error if it doesn't match.
         PrintError "Box Echo Error Detected"
      END IF
   END IF
END SUB


SUB SendTime (Frames&)       'Sends binary frame count to box, gets echo:
   TXData$ = MKL$(Frames&)   'Convert frames count to a four character string.
   PRINT #Box%, TXData$;                  'Send the string to the box.
   TimeoutCode% = 0
   TIMER ON
   ON TIMER(1) GOSUB NoResponse           'Set for 1 second timeout.
   DO WHILE (TimeoutCode% = 0) AND LOC(Box%) < 4  'Wait for echo or timeout.
      LOOP
   TIMER OFF
   IF NOT EOF(Box%) THEN
      Msg$ = INPUT$(LOC(Box%), #Box%)       'Read the entire message.
   ELSE
      Msg$ = ""
   END IF
   IF Msg$ <> TXData$ THEN                   'Error if it doesn't match.
      PrintError "Box Data Echo Error Detected"
   END IF
END SUB

